#include "spi.h"
#include "EPD_2in9.h"
#include <STC8H.H>
#include "delay.h"


unsigned char code EPD_2IN9_lut_full_update[] = {
    0x50, 0xAA, 0x55, 0xAA, 0x11, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0xFF, 0xFF, 0x1F, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00
};

unsigned char code EPD_2IN9_lut_partial_update[] = {
    0x10, 0x18, 0x18, 0x08, 0x18, 0x18,
    0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x13, 0x14, 0x44, 0x12,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00
};


void EPD_2IN9_Reset()
{
	RST=1;
	delay_us(5);
	RST=0;
	delay_us(5);
	RST=1;
	delay_us(5);
}


void EPD_2IN9_SendCmd(unsigned char dat)
{
	DC=0;
	CS=0;
	SPI_WriteByte(dat);
	CS=1;
}

void EPD_2IN9_SendData(unsigned char dat)
{
	DC=1;
	CS=0;
	SPI_WriteByte(dat);
	CS=1;
}


void EPD_2IN9_ReadBusy()
{
	while(BUSY);
}

void EPD_2IN9_SetWindows(unsigned short Xstart, unsigned short Ystart, unsigned short Xend, unsigned short Yend)
{
    EPD_2IN9_SendCmd(0x44); // SET_RAM_X_ADDRESS_START_END_POSITION
    EPD_2IN9_SendData((Xstart >> 3) & 0xFF);
    EPD_2IN9_SendData((Xend >> 3) & 0xFF);

    EPD_2IN9_SendCmd(0x45); // SET_RAM_Y_ADDRESS_START_END_POSITION
    EPD_2IN9_SendData(Ystart & 0xFF);
    EPD_2IN9_SendData((Ystart >> 8) & 0xFF);
    EPD_2IN9_SendData(Yend & 0xFF);
    EPD_2IN9_SendData((Yend >> 8) & 0xFF);
}

void EPD_2IN9_SetCursor(unsigned short Xstart, unsigned short Ystart)
{
    EPD_2IN9_SendCmd(0x4E); // SET_RAM_X_ADDRESS_COUNTER
    EPD_2IN9_SendData((Xstart >> 3) & 0xFF);

    EPD_2IN9_SendCmd(0x4F); // SET_RAM_Y_ADDRESS_COUNTER
    EPD_2IN9_SendData(Ystart & 0xFF);
    EPD_2IN9_SendData((Ystart >> 8) & 0xFF);
}




void EPD_2IN9_TurnOnDisplay(void)
{
    EPD_2IN9_SendCmd(0x22); // DISPLAY_UPDATE_CONTROL_2
    EPD_2IN9_SendData(0xC4);
    EPD_2IN9_SendCmd(0x20); // MASTER_ACTIVATION
    EPD_2IN9_SendCmd(0xFF); // TERMINATE_FRAME_READ_WRITE

    EPD_2IN9_ReadBusy();
}





void EPD_2IN9_Init(unsigned char Mode)
{
unsigned short i;
		delay(500);
		SPI_Init();
    EPD_2IN9_Reset();

    EPD_2IN9_SendCmd(0x01); // DRIVER_OUTPUT_CONTROL
    EPD_2IN9_SendData((EPD_2IN9_HEIGHT - 1) & 0xFF);
    EPD_2IN9_SendData(((EPD_2IN9_HEIGHT - 1) >> 8) & 0xFF);
    EPD_2IN9_SendData(0x00); // GD = 0; SM = 0; TB = 0;
	
    EPD_2IN9_SendCmd(0x0C); // BOOSTER_SOFT_START_CONTROL
    EPD_2IN9_SendData(0xD7);
    EPD_2IN9_SendData(0xD6);
    EPD_2IN9_SendData(0x9D);
	
    EPD_2IN9_SendCmd(0x2C); // WRITE_VCOM_REGISTER
    EPD_2IN9_SendData(0xA8); // VCOM 7C
	
    EPD_2IN9_SendCmd(0x3A); // SET_DUMMY_LINE_PERIOD
    EPD_2IN9_SendData(0x1A); // 4 dummy lines per gate
	
    EPD_2IN9_SendCmd(0x3B); // SET_GATE_TIME
    EPD_2IN9_SendData(0x08); // 2us per line
	
    EPD_2IN9_SendCmd(0x3C); // BORDER_WAVEFORM_CONTROL
    EPD_2IN9_SendData(0x03);                     
    EPD_2IN9_SendCmd(0x11); // DATA_ENTRY_MODE_SETTING
    EPD_2IN9_SendData(0x07);

    //set the look-up table register
    EPD_2IN9_SendCmd(0x32); // WRITE_LUT_REGISTER
    if(Mode == EPD_2IN9_FULL){
        for (i = 0; i < 30; i++) {
                EPD_2IN9_SendData(EPD_2IN9_lut_full_update[i]);
        }
    }else if(Mode == EPD_2IN9_PART){
        for (i = 0; i < 30; i++) {
                EPD_2IN9_SendData(EPD_2IN9_lut_partial_update[i]);
        }
    }else{
    }
}


void EPD_2IN9_Clear(void)
{
		unsigned short i,j;
    unsigned short Width, Height;
    Width = (EPD_2IN9_WIDTH % 8 == 0)? (EPD_2IN9_WIDTH / 8 ): (EPD_2IN9_WIDTH / 8 + 1);
    Height = EPD_2IN9_HEIGHT;
    EPD_2IN9_SetWindows(0, 0, EPD_2IN9_WIDTH, EPD_2IN9_HEIGHT);
    for (j = 0; j < Width; j++) {
        EPD_2IN9_SetCursor(j*8,0);
        EPD_2IN9_SendCmd(0x24);
        for (i = 0; i < Height; i++) {
            EPD_2IN9_SendData(0XFF);
        }
    }
    EPD_2IN9_TurnOnDisplay();
}


void EPD_2IN9_Display(unsigned char *Image)
{
		unsigned short i,j;
		unsigned short Addr = 0;
    unsigned short Width, Height;
    Width = (EPD_2IN9_WIDTH % 8 == 0)? (EPD_2IN9_WIDTH / 8 ): (EPD_2IN9_WIDTH / 8 + 1);
    Height = EPD_2IN9_HEIGHT;

    
    // UDOUBLE Offset = ImageName;
    EPD_2IN9_SetWindows(0, 0, EPD_2IN9_WIDTH, EPD_2IN9_HEIGHT);
    for (j = 0; j < Height; j++) {
        EPD_2IN9_SetCursor(0, j);
        EPD_2IN9_SendCmd(0x24);
        for (i = 0; i < Width; i++) {
            Addr = i + j * Width;
            EPD_2IN9_SendData(Image[Addr]);
        }
    }
    EPD_2IN9_TurnOnDisplay();
}


void EPD_2IN9_Sleep(void)
{
    EPD_2IN9_SendCmd(0x10);
    EPD_2IN9_SendData(0x01);
    EPD_2IN9_ReadBusy();
}
