/*-----------------------------------------------------------------------*/
/* Low level disk I/O module SKELETON for FatFs     (C)ChaN, 2019        */
/*-----------------------------------------------------------------------*/
/* If a working storage control module is available, it should be        */
/* attached to the FatFs via a glue function rather than modifying it.   */
/* This is an example of glue functions to attach various exsisting      */
/* storage control modules to the FatFs module with a defined API.       */
/*-----------------------------------------------------------------------*/

#include "ff.h"			/* Obtains integer types */
#include "diskio.h"		/* Declarations of disk functions */
#include "sd.h"


/*-----------------------------------------------------------------------*/
//ô״̬
/*-----------------------------------------------------------------------*/
DSTATUS disk_status ( BYTE pdrv )
{
	switch (pdrv) 
    {
        case SD :
                return RES_OK; 
            break;
        default:
            break;
	}
	return STA_NOINIT;
}


/*-----------------------------------------------------------------------*/
//ʼ
/*-----------------------------------------------------------------------*/
DSTATUS disk_initialize ( BYTE pdrv )
{
	switch (pdrv) 
    {
        case SD :
								SD_Init();
                return RES_OK;
            break;
        default:
            break;
	}
	return STA_NOINIT;
}

/*-----------------------------------------------------------------------*/
//
//pdrv:̱0~9
//*buff:ݽջ׵ַ
//sector:ַ
//count:Ҫȡ
/*-----------------------------------------------------------------------*/
DRESULT disk_read (	BYTE pdrv, BYTE *buff, LBA_t sector, UINT count )
{
    if (!count)return RES_PARERR;

	switch (pdrv) 
    {
        case SD :
            while(count--){
                SD_ReadBlock(buff,sector);
                sector++;
                buff+=512;
            }
            return RES_OK;
        default:
            break;
	}
	return RES_PARERR;
}


/*-----------------------------------------------------------------------*/
//д
//pdrv:̱0~9
//*buff:׵ַ
//sector:ַ
//count:Ҫд
/*-----------------------------------------------------------------------*/
#if FF_FS_READONLY == 0

DRESULT disk_write ( BYTE pdrv, const BYTE *buff, LBA_t sector, UINT count )
{
	switch (pdrv) 
    {
        case SD :
            while(count--){
                SD_WriteBlock(buff,sector);
                sector++;
                buff+=512;
            }
            return RES_OK;
        default:
            break;
	}
	return RES_PARERR;
}

#endif


/*-----------------------------------------------------------------------*/
//Ļ
//pdrv:̱0~9
//cmd:ƴ
//*buff:/ջָ
/*-----------------------------------------------------------------------*/
DRESULT disk_ioctl ( BYTE pdrv, BYTE cmd, void *buff )
{
    unsigned char *pt = buff;
	switch (pdrv) 
    {
        case SD:
            switch (cmd)
            {
                case CTRL_SYNC:
                    return RES_OK; 
                case GET_SECTOR_COUNT:
										*(DWORD*)buff=SD_GetDriverSize();										//
                    return RES_OK;
                case GET_SECTOR_SIZE:
                    *(WORD*)buff = 512;                                //С
                    return RES_OK;
                case GET_BLOCK_SIZE:
                    *(DWORD*)buff = 1;                                 //ΪλĴشС, ĬΪ1, ʽʱʹ.
                    return RES_OK;
            }
	}
	return RES_PARERR;
}

//ʱ
//User defined function to give a current time to fatfs module      */
//31-25: Year(0-127 org.1980), 24-21: Month(1-12), 20-16: Day(1-31) */                                                                                                                                                                                                                                          
//15-11: Hour(0-23), 10-5: Minute(0-59), 4-0: Second(0-29 *2) */                                                                                                                                                                                                                                                
DWORD get_fattime (void)
{				 
	return 0;
}		

